/*
 ATtiny13 LiFePO4 Charger
 Version: 1.0
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 3/02/2018
 Last Modified: 19/03/2018
  
 */
 
#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif

#ifndef NULL
#define NULL ((void *)0)
#endif

#define LOW 0
#define HIGH 1
#define false 0
#define true 1

#define T16MS 0
#define T32MS 1
#define T64MS 2
#define T128MS 3
#define T250MS 4
#define T500MS 5
#define T1S 6
#define T2S 7
#define T4S 8
#define T8S 9

#define ADC_REF_VCC 0
#define ADC_REF_1_1V 1

// Put the ATtiny in power down sleep mode and switch off the ADC too
void system_sleep(void) {
	set_sleep_mode(SLEEP_MODE_PWR_DOWN); // Set sleep mode
	sleep_mode(); // System sleeps here
}

// Watchdog timeout values
// 0=16ms, 1=32ms, 2=64ms, 3=128ms, 4=250ms, 5=500ms, 6=1sec, 7=2sec, 8=4sec, 9=8sec
void setup_watchdog(uint8_t wdDelay) {
	if (wdDelay > 9) { // Set max as 8 sec
		wdDelay = 9;
	}
	if (wdDelay > 7) { // Set WDP3 bit
		wdDelay = wdDelay & 7;
		wdDelay |= (1<<5);
	}
	
	MCUSR &= ~(1<<WDRF);
	// Start timed sequence
	WDTCR |= (1<<WDCE) | (1<<WDE);
	// Set new watchdog timeout value
	WDTCR = wdDelay | (1<<WDTIE);
}

// Turn off the Watchdog
void turn_off_watchdog(void) {
	cli(); // Turn interrupts off
	MCUSR &= ~(1<<WDRF);
	WDTCR |= (1<<WDCE) | (1<<WDE);
	WDTCR = 0;
	sei(); // Turn interrupts back on
}

// Watchdog sleep function - combines all the above functions into one
void watchdog_sleep(uint8_t timeout) { 
	setup_watchdog(timeout);
	system_sleep();
	turn_off_watchdog();
}

// ADC Read
int adc_read(uint8_t reference, uint8_t pin) {
	uint8_t low, high;
	
	// Set the pin and ref
	if (reference == ADC_REF_1_1V) {
		ADMUX = (1<<REFS0) | (pin & 0x3F);
	}
	else {
		ADMUX = (pin & 0x3F);
	}
	
	set_sleep_mode(SLEEP_MODE_ADC); // Set sleep mode ADC
	sleep_mode(); // System sleeps here
	
	// we have to read ADCL first; doing so locks both ADCL
	// and ADCH until ADCH is read.  reading ADCL second would
	// cause the results of each conversion to be discarded,
	// as ADCL and ADCH would be locked when it completed.
	low = ADCL;
	high = ADCH;
	
	// combine the two bytes
	return (high << 8) | low;
}

// Setup the ATtiny
void setup(void) {
	// Set outputs
	DDRB |= (1<<chargingMosfet) | (1<<sensingVoltageMosfet);
	PORTB &= ~((1<<chargingMosfet) | (1<<sensingVoltageMosfet)); // Turn off mosfets
	
	// Turn on pull-up resistors on other ports to save power
	PORTB |= (1<<PB0);
	
	// Enable pin change interrupt
	sbi(GIMSK, PCIE);
	
	// Set ADC prescale factor to 8
	// 1 MHz / 8 = 125 KHz, inside the desired 50-200 KHz range.
	sbi(ADCSRA, ADPS1);
	sbi(ADCSRA, ADPS0);
	
	// Turn on ADC interrupt
	sbi(ADCSRA, ADIE);
	
	sei(); // Turn on interrupts
}
